// ----------------------------------
// Filename      : NetworkViewer.java
// Created       : 11/10/1999
// Last modified : 26/04/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------

import mitrasim.exceptions.*;
import mitrasim.network.*;
import mitrasim.resources.*;
import mitrasim.util.*;
import mitrasim.viewer.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class NetworkViewer
{
  private static String fLanguageFileName;
  private static String fNetworkFileName;

  public static void main(String[] argv)
  {
    // enable access to the resources for Mitrasim 2000
    Resources fResources = new Resources();
    fResources.loadConfiguration();

    try {
      parseCommandLine(argv);

      // access the language-file
      fResources.fMessages = new Messages(fLanguageFileName);
      System.out.println(fResources.fMessages.lookup("messageStartupBanner"));
      System.out.println("===> Network Viewer <===");
      System.out.println("");
      System.out.println(fResources.fMessages.lookup("messageThisLanguage"));
      System.out.println("");

      // access the network-file
      fResources.fNetwork = new Network(fResources.fMessages);
      fResources.fNetwork.loadFromFile(fNetworkFileName);

      // check the network
      System.out.println(fResources.fMessages.lookup("messageCheckingNetwork"));
      Report ResultReport = fResources.fNetwork.analyse();
      if (ResultReport.isEmpty()) {
        System.out.println(fResources.fMessages.lookup("messageNetworkComplies"));
        System.out.println("");
      }
      else {
        System.out.println(fResources.fMessages.lookup("errorNetworkDoesntComply"));
        System.out.println("");
        ResultReport.initializeReadOut();
        String ErrorMessage = ResultReport.getNextLine();
        while (!ErrorMessage.equalsIgnoreCase("")) {
          System.out.println(" => " + ErrorMessage);
          ErrorMessage = ResultReport.getNextLine();
        }
        System.out.println("");
        System.exit(0);
      }

      fResources.fNetwork.calcGeometry();

      // find the amount of available memory
      Runtime runtime = Runtime.getRuntime ();
      runtime.gc (); // run garbage-collector
      long FreeMemory = runtime.freeMemory();
      long TotalMemory = runtime.totalMemory();
      System.out.println(fResources.fMessages.lookup("messageReportFreeMemory",String.valueOf(FreeMemory / 1024)));
      System.out.println(fResources.fMessages.lookup("messageReportTotalMemory",String.valueOf(TotalMemory / 1024)));
      System.out.println("");

      // launch the GUI
      System.out.println(fResources.fMessages.lookup("messageLaunchingGUI"));
      ViewerGUI viewerGUI = new ViewerGUI();
    }
    catch (GeneralError err) {
      switch (err.errorCategory()) {
        case GeneralError.kFileError : System.out.println("File error :");
                                       break;
        case GeneralError.kParseError : System.out.println("Parse error :");
                                        break;
        case GeneralError.kNetworkError : System.out.println("Network error :");
                                          break;
        case GeneralError.kCmdLineError : System.out.println("Commandline error :");
                                          break;
      }
      System.out.println(" => " + err.message());
    }
  }

  private static void parseCommandLine(String[] argv) throws GeneralError
  {
    // initialize the default filenames
    fLanguageFileName = "language-english.data";
    fNetworkFileName = "network.data";

    if (argv.length > 0) {
      for (int i = 0; i < argv.length; ++i) {
        String fOption = argv[i];

        if (fOption.startsWith("--help")) {
          System.out.println("");
          System.out.println("Usage : NetworkViewer [options]");
          System.out.println("");
          System.out.println("Valid options are : --help");
          System.out.println("                    --language=Dutch|English");
          System.out.println("                    --network=networkfile");
          System.out.println("");
          System.out.println("If no language is specified, then English is assumed.");
          System.out.println("If no networkfile is specified, then network.data is assumed.");
          System.exit(0);
        }
        else if (fOption.startsWith("--language=")) {
          // cut the option, leaving the language
          fOption = fOption.substring(11);
          if (fOption.equalsIgnoreCase("English")) {
            fLanguageFileName = "language-english.data";
          }
          else {
            if (fOption.equalsIgnoreCase("Dutch")) {
              fLanguageFileName = "language-dutch.data";
            }
            else {
              throw (new GeneralError("Invalid language [" + fOption + "] specified.",
                                      GeneralError.kCmdLineError));
            }
          }
        }
        else if (fOption.startsWith("--network=")) {
          // cut the option, leaving the networkfile
          fNetworkFileName = fOption.substring(10);
        }
        else {
          throw (new GeneralError("Unknown or incomplete option [" + fOption + "] specified.",
                                  GeneralError.kCmdLineError));
        }
      }
    }
  }
}
