// -----------------------------------
// Filename      : EnvironmentBox.java
// Created       : 08/02/2000
// Last modified : 15/06/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// -----------------------------------

package mitrasim.modeler;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import mitrasim.network.*;
import mitrasim.resources.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class EnvironmentBox extends Dialog implements WindowListener, ActionListener
{
  // the different dialog-modes
  public static final int kAddMode  = 1;
  public static final int kEditMode = 2;

  // internal datastructures
  private Resources        fResources;
  private Messages         fMessages;
  private int              fDialogMode;
  private Environment      fEnvironment;
  private final int        kType          = 1;
  private final int        kColor         = 2;
  private final int        kFilled        = 3;
  private final int        kNodes         = 4;
  private final int        kXRadius       = 5;
  private final int        kYRadius       = 6;
  private final int        kName          = 7;
  private CfgItemList      fCfgType;
  private Label            fLabelColor;
  private CfgItemList      fCfgColor;
  private Label            fLabelFilled;
  private CfgCheckBox      fCfgFilled;
  private CfgNodesInputBox fCfgNodes;
  private Label            fLabelNodes;
  private Label            fLabelXRadius;
  private CfgInputBox      fCfgXRadius;
  private Label            fLabelXRadiusDistanceUnit;
  private Label            fLabelYRadius;
  private CfgInputBox      fCfgYRadius;
  private Label            fLabelYRadiusDistanceUnit;
  private CfgInputBox      fCfgName;
  private Button           fOkButton;
  private boolean          fCanceled;
  private boolean          fFieldsDefined;

  public EnvironmentBox(Frame parent, int DialogMode, Environment environment)
  {
    super(parent);

    fResources = new Resources();
    fMessages = fResources.fMessages;
    fDialogMode = DialogMode;
    fEnvironment = environment;
    fCanceled = true;
    fFieldsDefined = false;

    // adjust the windowlayout
    setFont(new Font(fMessages.lookup("textFontName"),Font.PLAIN,14));
    setResizable(false);
    Point Org = parent.getLocation();
    setLocation(Org.x + 20,Org.y + 60);
    if (fDialogMode == kAddMode) {
      setTitle(fMessages.lookup("textAddEnvironmentDialogTitle"));
    }
    else if (fDialogMode == kEditMode) {
      setTitle(fMessages.lookup("textEditEnvironmentDialogTitle"));
    }
    setModal(true);
    setBackground(Color.lightGray);
    setForeground(Color.black);

    // define the controlpanels
    Panel SettingsPanel = new Panel();
    Panel ControlPanel = new Panel();
    SettingsPanel.setLayout(new GridLayout(11,3));
    ControlPanel.setLayout(new GridLayout(1,2));

    SettingsPanel.add(new Label(fMessages.lookup("textEnvironmentType") + " = ",Label.RIGHT));
    fCfgType = new CfgItemList(kType);
    SettingsPanel.add(fCfgType);
    SettingsPanel.add(new Label());

    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());

    fLabelColor = new Label(fMessages.lookup("textEnvironmentColor") + " = ",Label.RIGHT);
    SettingsPanel.add(fLabelColor);
    fCfgColor = new CfgItemList(kColor);
    SettingsPanel.add(fCfgColor);
    SettingsPanel.add(new Label());

    fLabelFilled = new Label(fMessages.lookup("textEnvironmentFilled") + " = ",Label.RIGHT);
    SettingsPanel.add(fLabelFilled);
    fCfgFilled = new CfgCheckBox(kFilled);
    SettingsPanel.add(fCfgFilled);
    SettingsPanel.add(new Label());

    SettingsPanel.add(new Label(fMessages.lookup("textEnvironmentNodes") + " = ",Label.RIGHT));
    fCfgNodes = new CfgNodesInputBox();
    SettingsPanel.add(fCfgNodes);
    fLabelNodes = new Label(" (# = )",Label.LEFT);
    SettingsPanel.add(fLabelNodes);

    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());

    fLabelXRadius = new Label(fMessages.lookup("textEnvironmentXRadius") + " = ",Label.RIGHT);
    SettingsPanel.add(fLabelXRadius);
    fCfgXRadius = new CfgInputBox(kXRadius);
    SettingsPanel.add(fCfgXRadius);
    fLabelXRadiusDistanceUnit = new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT);
    SettingsPanel.add(fLabelXRadiusDistanceUnit);

    fLabelYRadius = new Label(fMessages.lookup("textEnvironmentYRadius") + " = ",Label.RIGHT);
    SettingsPanel.add(fLabelYRadius);
    fCfgYRadius = new CfgInputBox(kYRadius);
    SettingsPanel.add(fCfgYRadius);
    fLabelYRadiusDistanceUnit = new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT);
    SettingsPanel.add(fLabelYRadiusDistanceUnit);

    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());

    SettingsPanel.add(new Label(fMessages.lookup("textEnvironmentName") + " = ",Label.RIGHT));
    fCfgName = new CfgInputBox(kName);
    SettingsPanel.add(fCfgName);
    SettingsPanel.add(new Label());

    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());

    Button button;
    button = new Button(fMessages.lookup("buttonCancel"));
    button.addActionListener(this);
    ControlPanel.add(button);

    fOkButton = new Button(fMessages.lookup("buttonOk"));
    fOkButton.addActionListener(this);
    ControlPanel.add(fOkButton);

    // add the controlpanel
    setLayout(new BorderLayout());
    add(SettingsPanel,"North");
    add(ControlPanel,"South");

    // add the windowlistener
    addWindowListener(this);

    fFieldsDefined = true;
    updateAllFields();

    pack();
    show();
  }

  public boolean canceled()
  {
    return fCanceled;
  }

  public Environment getEnvironment()
  {
    return fEnvironment;
  }

  private void updateAllFields()
  {
    fCfgType.update();
    fCfgColor.update();
    fCfgFilled.update();
    fCfgNodes.update();
    fCfgXRadius.update();
    fCfgYRadius.update();
    fCfgName.update();

    if (fEnvironment.fType == Environment.kRectangle) {
      if (fEnvironment.fNrOfNodes != 2) {
        fOkButton.setEnabled(false);
      }
      else {
        fOkButton.setEnabled(true);
      }
    }
    else if (fEnvironment.fType == Environment.kPolygon) {
      if (fEnvironment.fNrOfNodes < 3) {
        fOkButton.setEnabled(false);
      }
      else {
        fOkButton.setEnabled(true);
      }
    }
    else if (fEnvironment.fType == Environment.kOval) {
      if (fEnvironment.fNrOfNodes != 1) {
        fOkButton.setEnabled(false);
      }
      else {
        fOkButton.setEnabled(true);
      }
    }
    else if (fEnvironment.fType == Environment.kRailroad) {
      if (fEnvironment.fNrOfNodes < 2) {
        fOkButton.setEnabled(false);
      }
      else {
        fOkButton.setEnabled(true);
      }
    }

    // enable or disable the appropriate fields
    if ((fEnvironment.fType == Environment.kRectangle) ||
        (fEnvironment.fType == Environment.kPolygon) ||
        (fEnvironment.fType == Environment.kOval)) {
      fLabelColor.setEnabled(true);
      fCfgColor.setEnabled(true);
      fLabelFilled.setEnabled(true);
      fCfgFilled.setEnabled(true);
    }
    else if (fEnvironment.fType == Environment.kRailroad) {
      fLabelColor.setEnabled(false);
      fCfgColor.setEnabled(false);
      fLabelFilled.setEnabled(false);
      fCfgFilled.setEnabled(false);
    }

    if ((fEnvironment.fType == Environment.kRectangle) ||
        (fEnvironment.fType == Environment.kPolygon) ||
        (fEnvironment.fType == Environment.kRailroad)) {
      fLabelXRadius.setEnabled(false);
      fCfgXRadius.setEnabled(false);
      fLabelXRadiusDistanceUnit.setEnabled(false);
      fLabelYRadius.setEnabled(false);
      fCfgYRadius.setEnabled(false);
      fLabelYRadiusDistanceUnit.setEnabled(false);
    }
    else if (fEnvironment.fType == Environment.kOval) {
      fLabelXRadius.setEnabled(true);
      fCfgXRadius.setEnabled(true);
      fLabelXRadiusDistanceUnit.setEnabled(true);
      fLabelYRadius.setEnabled(true);
      fCfgYRadius.setEnabled(true);
      fLabelYRadiusDistanceUnit.setEnabled(true);
    }
  }

  // methods from the WindowListener-interface
  public void windowOpened(WindowEvent e) { }
  public void windowClosing(WindowEvent e)
  {
    dispose();
  }
  public void windowClosed(WindowEvent e) { }
  public void windowIconified(WindowEvent e) { }
  public void windowDeiconified(WindowEvent e) { }
  public void windowActivated(WindowEvent e) { }
  public void windowDeactivated(WindowEvent e) { }

  // method from the ActionListener-interface
  public void actionPerformed(ActionEvent e)
  {
    String Command = e.getActionCommand();

    if (Command.equals(fMessages.lookup("buttonCancel"))) {
      windowClosing(null);
    }

    if (Command.equals(fMessages.lookup("buttonOk"))) {
      fCanceled = false;
      windowClosing(null);
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgInputBox extends TextField implements ActionListener
  {
    private int fInputBox;

    public CfgInputBox(int inputBox)
    {
      super();
      fInputBox = inputBox;
      update();
      addActionListener(this);
    }

    synchronized public void update()
    {
      if (fInputBox == kXRadius) {
        setText(String.valueOf(fEnvironment.fXRadius));
      }
      else if (fInputBox == kYRadius) {
        setText(String.valueOf(fEnvironment.fYRadius));
      }
      else if (fInputBox == kName) {
        setText(String.valueOf(fEnvironment.fName));
      }
    }

    // method from the ActionListener-interface
    public void actionPerformed(ActionEvent e)
    {
      int IntInput = 0;
      float FloatInput = 0.0f;
      String StringInput = "";
      try {
        if ((fInputBox == kXRadius) || (fInputBox == kYRadius)) {
          IntInput = (new Integer(e.getActionCommand())).intValue();
        }
        else if (fInputBox == kName) {
          StringInput = e.getActionCommand();
        }
      }
      catch (NumberFormatException exc) {
        if (fInputBox == kXRadius) {
          IntInput = fEnvironment.fXRadius;
        }
        else if (fInputBox == kYRadius) {
          IntInput = fEnvironment.fYRadius;
        }
      }

      if (fInputBox == kXRadius) {
        fEnvironment.fXRadius = IntInput;
      }
      else if (fInputBox == kYRadius) {
        fEnvironment.fYRadius = IntInput;
      }
      else if (fInputBox == kName) {
        fEnvironment.fName = StringInput;
      }

      updateAllFields();
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgNodesInputBox extends TextField implements ActionListener
  {
    public CfgNodesInputBox()
    {
      super();
      update();
      addActionListener(this);
    }

    synchronized public void update()
    {
      // concatenate all the node-IDs
      String AllIDs = "";
      for (int i = 0; i < fEnvironment.fNrOfNodes; ++i) {
        AllIDs = AllIDs.concat(String.valueOf(fEnvironment.fNodes[i].fID));
        if (i < (fEnvironment.fNrOfNodes - 1)) {
          AllIDs = AllIDs.concat(",");
        }
      }
      setText(AllIDs);
      if (fFieldsDefined) {
        fLabelNodes.setText(" (# = " + String.valueOf(fEnvironment.fNrOfNodes) + ")");
      }
    }

    // method from the ActionListener-interface
    public void actionPerformed(ActionEvent e)
    {
      String Input = e.getActionCommand();
      String Result = "";

      // to temporary store the IDs
      Vector Nodes = new Vector();

      String ID = "";
      // parse the ID-string and extract all IDs
      for (int Index = 0; Index < Input.length(); ++Index) {
        char ch = Input.charAt(Index);
        if (ch != ',') {
          ID = ID.concat(String.valueOf(ch));
        }
        else {
          // we have completed extraction of one ID
          // test if it is a valid number
          try {
            int NodeID = (new Integer(ID)).intValue();
            // test if the specified node exists
            if (fResources.fNetwork.fNodes.get(new Integer(NodeID)) != null) {
              // add the ID to the result
              Result = Result.concat(ID + ",");
              // add the ID to the list of nodes
              Nodes.addElement(new Integer(ID));
            }
            else {
              // the specified node doesn't exist so we ignore it
            }
          }
          catch (NumberFormatException exc) {
            // ignore the ID
          }
          ID = "";
        }
      }

      // we have completed extraction of the last ID
      // test if it is a valid number
      try {
        int NodeID = (new Integer(ID)).intValue();
        // test if the specified node exists
        if (fResources.fNetwork.fNodes.get(new Integer(NodeID)) != null) {
          // add the ID to the result
          Result = Result.concat(ID + ",");
          // add the ID to the list of nodes
          Nodes.addElement(new Integer(ID));
        }
        else {
          // the specified node doesn't exist so we ignore it
        }
      }
      catch (NumberFormatException exc) {
        // ignore the ID
      }

      // delete the final comma
      if (Result.endsWith(",")) {
        Result = Result.substring(0,Result.length() - 1);
      }

      // modify the Nodes[]-field in fEnvironment
      fEnvironment.fNodes = new Node[Nodes.size()];
      fEnvironment.fNrOfNodes = Nodes.size();
      Enumeration NodeEnum = Nodes.elements();
      int NodeIndex = 0;
      while (NodeEnum.hasMoreElements()) {
        fEnvironment.fNodes[NodeIndex] = (Node) fResources.fNetwork.fNodes.get((Integer) NodeEnum.nextElement());
        ++NodeIndex;
      }

      updateAllFields();
    }

    public Dimension getPreferredSize()
    {
      return new Dimension(150,25);
    }

    public Dimension getMinimumSize()
    {
      return getPreferredSize();
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgCheckBox extends Checkbox implements ItemListener
  {
    private int fCheckBox;

    public CfgCheckBox(int checkBox)
    {
      super();
      fCheckBox = checkBox;
      addItemListener(this);
      update();
    }

    synchronized public void update()
    {
      if (fCheckBox == kFilled) {
        setState(fEnvironment.fFilled);
        if (fEnvironment.fFilled) {
          setLabel("(" + fMessages.lookup("textYes") + ")");
        }
        else {
          setLabel("(" + fMessages.lookup("textNoOutlined") + ")");
        }
      }
    }

    // method from the ItemListener-interface
    public void itemStateChanged(ItemEvent e)
    {
      if (fCheckBox == kFilled) {
        fEnvironment.fFilled = !fEnvironment.fFilled;
      }

      updateAllFields();
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgItemList extends Choice implements ItemListener
  {
    private int fItemList;

    public CfgItemList(int itemList)
    {
      super();
      fItemList = itemList;
      if (fItemList == kType) {
        add(fMessages.lookup("textRectangleEnvironment"));
        add(fMessages.lookup("textPolygonEnvironment"));
        add(fMessages.lookup("textOvalEnvironment"));
        add(fMessages.lookup("textRailroadEnvironment"));
      }
      else if (fItemList == kColor) {
        add(fMessages.lookup("textColorBlack"));
        add(fMessages.lookup("textColorBlue"));
        add(fMessages.lookup("textColorCyan"));
        add(fMessages.lookup("textColorGray"));
        add(fMessages.lookup("textColorGreen"));
        add(fMessages.lookup("textColorMagenta"));
        add(fMessages.lookup("textColorOrange"));
        add(fMessages.lookup("textColorRed"));
        add(fMessages.lookup("textColorWhite"));
        add(fMessages.lookup("textColorYellow"));
      }
      update();
      addItemListener(this);
    }

    synchronized public void update()
    {
      if (fItemList == kType) {
        if (fEnvironment.fType == Environment.kRectangle) {
          select(fMessages.lookup("textRectangleEnvironment"));
        }
        else if (fEnvironment.fType == Environment.kPolygon) {
          select(fMessages.lookup("textPolygonEnvironment"));
        }
        else if (fEnvironment.fType == Environment.kOval) {
          select(fMessages.lookup("textOvalEnvironment"));
        }
        else if (fEnvironment.fType == Environment.kRailroad) {
          select(fMessages.lookup("textRailroadEnvironment"));
        }
      }
      else if (fItemList == kColor) {
        if (fEnvironment.fColor == Color.black) {
          select(fMessages.lookup("textColorBlack"));
        }
        else if (fEnvironment.fColor == Color.blue) {
          select(fMessages.lookup("textColorBlue"));
        }
        else if (fEnvironment.fColor == Color.cyan) {
          select(fMessages.lookup("textColorCyan"));
        }
        else if (fEnvironment.fColor == Color.gray) {
          select(fMessages.lookup("textColorGray"));
        }
        else if (fEnvironment.fColor == Color.green) {
          select(fMessages.lookup("textColorGreen"));
        }
        else if (fEnvironment.fColor == Color.magenta) {
          select(fMessages.lookup("textColorMagenta"));
        }
        else if (fEnvironment.fColor == Color.orange) {
          select(fMessages.lookup("textColorOrange"));
        }
        else if (fEnvironment.fColor == Color.red) {
          select(fMessages.lookup("textColorRed"));
        }
        else if (fEnvironment.fColor == Color.white) {
          select(fMessages.lookup("textColorWhite"));
        }
        else if (fEnvironment.fColor == Color.yellow) {
          select(fMessages.lookup("textColorYellow"));
        }
      }
    }

    // method from the ItemListener-interface
    public void itemStateChanged(ItemEvent e)
    {
      if (fItemList == kType) {
        String EnvironmentType = (String) e.getItem();

        if (EnvironmentType.equalsIgnoreCase(fMessages.lookup("textRectangleEnvironment"))) {
          fEnvironment.fType = Environment.kRectangle;
        }
        else if (EnvironmentType.equalsIgnoreCase(fMessages.lookup("textPolygonEnvironment"))) {
          fEnvironment.fType = Environment.kPolygon;
        }
        else if (EnvironmentType.equalsIgnoreCase(fMessages.lookup("textOvalEnvironment"))) {
          fEnvironment.fType = Environment.kOval;
        }
        else if (EnvironmentType.equalsIgnoreCase(fMessages.lookup("textRailroadEnvironment"))) {
          fEnvironment.fType = Environment.kRailroad;
        }
      }
      else if (fItemList == kColor) {
        String EnvironmentColor = (String) e.getItem();

        if (EnvironmentColor.equalsIgnoreCase(fMessages.lookup("textColorBlack"))) {
          fEnvironment.fColor = Color.black;
        }
        else if (EnvironmentColor.equalsIgnoreCase(fMessages.lookup("textColorBlue"))) {
          fEnvironment.fColor = Color.blue;
        }
        else if (EnvironmentColor.equalsIgnoreCase(fMessages.lookup("textColorCyan"))) {
          fEnvironment.fColor = Color.cyan;
        }
        else if (EnvironmentColor.equalsIgnoreCase(fMessages.lookup("textColorGray"))) {
          fEnvironment.fColor = Color.gray;
        }
        else if (EnvironmentColor.equalsIgnoreCase(fMessages.lookup("textColorGreen"))) {
          fEnvironment.fColor = Color.green;
        }
        else if (EnvironmentColor.equalsIgnoreCase(fMessages.lookup("textColorMagenta"))) {
          fEnvironment.fColor = Color.magenta;
        }
        else if (EnvironmentColor.equalsIgnoreCase(fMessages.lookup("textColorOrange"))) {
          fEnvironment.fColor = Color.orange;
        }
        else if (EnvironmentColor.equalsIgnoreCase(fMessages.lookup("textColorRed"))) {
          fEnvironment.fColor = Color.red;
        }
        else if (EnvironmentColor.equalsIgnoreCase(fMessages.lookup("textColorWhite"))) {
          fEnvironment.fColor = Color.white;
        }
        else if (EnvironmentColor.equalsIgnoreCase(fMessages.lookup("textColorYellow"))) {
          fEnvironment.fColor = Color.yellow;
        }
      }

      updateAllFields();
    }
  }
}
