// ----------------------------------
// Filename      : MLegend.java
// Created       : 30/01/2000
// Last modified : 03/03/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------

package mitrasim.modeler;

import java.awt.*;
import java.awt.event.*;
import mitrasim.resources.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class MLegend extends Frame
{
  // internal datastructures
  private Resources    fResources;
  private Messages     fMessages;
  private LegendCanvas fLegendCanvas;

  // only allow one instance of this class
  private static boolean fInstanceCreated;

  static {
    fInstanceCreated = false;
  }

  public MLegend()
  {
    super();

    if (fInstanceCreated) {
      return;
    }
    fInstanceCreated = true;

    fResources = new Resources();
    fMessages = fResources.fMessages;

    // adjust the windowlayout
    setFont(new Font(fMessages.lookup("textFontName"),Font.PLAIN,14));
    setSize(fResources.fnmLegendXSize,fResources.fnmLegendYSize);
    setResizable(false);
    setLocation(100,100);
    setTitle(fMessages.lookup("textLegendWindowTitle"));
    setBackground(Color.lightGray);
    setForeground(Color.black);
    setIconImage(Toolkit.getDefaultToolkit().getImage("M2000Icon.jpg"));

    setLayout(new BorderLayout());
    fLegendCanvas = new LegendCanvas(fResources.fnmLegendXSize - 20,
                                     fResources.fnmLegendYSize - 20);
    add("Center",fLegendCanvas);

    pack();
    show();
  }

  public void repaintLegend()
  {
    fLegendCanvas.repaint();
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  // the message-canvas
  private class LegendCanvas extends Canvas
  {
    // the canvas-draw-area
    private int cx1;
    private int cy1;
    private int cx2;
    private int cy2;
    private int cwidth;
    private int cheight;

    public LegendCanvas(int width, int height)
    {
      super();

      setSize(width,height);
    }

    public void update(Graphics g)
    {
      paint(g);
    }

    public void paint(Graphics g)
    {
      // define the boundaries for drawing
      cx1 = 5;
      cy1 = 5;
      cx2 = (cx1 + getSize().width) - (getInsets().right + 5 + 5);
      cy2 = (cy1 + getSize().height) - (getInsets().bottom + 5 + 5);
      cwidth = cx2 - cx1;
      cheight = cy2 - cy1;

      // activate clipping of graphics primitives
      g.setClip(cx1,cy1,cwidth,cheight);

      // clear the canvas
      g.setColor(Color.lightGray);
      g.fillRect(cx1,cy1,cwidth,cheight);

      // show the different node-types
      g.setColor(Color.black);
      g.drawString(fMessages.lookup("textNodesCaption"),cx1 + 10,cy1 + 20);
      g.drawString(fMessages.lookup("textNodesCaption"),cx1 + 11,cy1 + 20);
      showNode(g,fResources.knmOriginNodeColor,cy1 + 40,fMessages.lookup("textOriginNode"));
      showNode(g,fResources.knmDestinationNodeColor,cy1 + 60,fMessages.lookup("textDestinationNode"));
      showNode(g,fResources.knmJunctionNodeColor,cy1 + 80,fMessages.lookup("textJunctionNode"));
      showNode(g,fResources.knmBendNodeColor,cy1 + 100,fMessages.lookup("textBendNode"));
      showNode(g,fResources.knmEnvironmentNodeColor,cy1 + 120,fMessages.lookup("textEnvironmentNode"));

      // show the different link-types
      g.setColor(Color.black);
      g.drawString(fMessages.lookup("textLinksCaption"),cx1 + 170,cy1 + 20);
      g.drawString(fMessages.lookup("textLinksCaption"),cx1 + 171,cy1 + 20);

      int LastYOffset = 0;
      if (fResources.fnmColorLinksAccordingToType) {
        showLink(g,fResources.knmTunnelLinkColor,cy1 + 40,fMessages.lookup("textTunnelLink"));
        showLink(g,fResources.knmHighwayLinkColor,cy1 + 60,fMessages.lookup("textHighwayLink"));
        showLink(g,fResources.knmSeparatedOnRampLinkColor,cy1 + 80,fMessages.lookup("textSeparatedOnRampLink"));
        showLink(g,fResources.knmMergedOnRampLinkColor,cy1 + 100,fMessages.lookup("textMergedOnRampLink"));
        showLink(g,fResources.knmSeparatedOffRampLinkColor,cy1 + 120,fMessages.lookup("textSeparatedOffRampLink"));
        showLink(g,fResources.knmMergedOffRampLinkColor,cy1 + 140,fMessages.lookup("textMergedOffRampLink"));
        showLink(g,fResources.knmNarrowingLinkColor,cy1 + 160,fMessages.lookup("textNarrowingLink"));
        showLink(g,fResources.knmWideningLinkColor,cy1 + 180,fMessages.lookup("textWideningLink"));
        LastYOffset = 180;
      }
      else {
        showLink(g,fResources.knm0To19SpeedColor,cy1 +  40,fMessages.lookup("textLessThan20Kmh"));
        showLink(g,fResources.knm20To39SpeedColor,cy1 +  60,fMessages.lookup("text20To39Kmh"));
        showLink(g,fResources.knm40To59SpeedColor,cy1 +  80,fMessages.lookup("text40To59Kmh"));
        showLink(g,fResources.knm60To79SpeedColor,cy1 + 100,fMessages.lookup("text60To79Kmh"));
        showLink(g,fResources.knm80To99SpeedColor,cy1 + 120,fMessages.lookup("text80To99Kmh"));
        showLink(g,fResources.knm100To119SpeedColor,cy1 + 140,fMessages.lookup("text100To119Kmh"));
        showLink(g,fResources.knm120AndUpSpeedColor,cy1 + 160,fMessages.lookup("text120AndUpKmh"));
        LastYOffset = 160;
      }

      // show the railroad-environment
      showRailroadEnvironment(g,cy1 + LastYOffset + 20);
    }

    private void showNode(Graphics g, Color NodeColor, int y, String NodeType)
    {
      g.setColor(NodeColor);
      g.fillOval(cx1 + 10,y,10,10);
      g.setColor(Color.black);
      g.drawOval(cx1 + 10,y,10,10);
      g.drawString(NodeType,cx1 + 30,y + 10);
    }

    private void showLink(Graphics g, Color LinkColor, int y, String Description)
    {
      g.setColor(LinkColor);
      g.fillRect(cx1 + 170,y - 2,20,5);
      g.setColor(Color.black);
      g.drawString(Description,cx1 + 200,y + 5);
    }

    private void showRailroadEnvironment(Graphics g, int y)
    {
      g.setColor(Color.white);
      g.drawLine(cx1 + 170,y - 2,cx1 + 190,y - 2);

      g.setColor(Color.black);
      g.fillRect(cx1 + 170,y - 1,20,3);

      g.setColor(Color.white);
      g.drawLine(cx1 + 170,y + 2,cx1 + 190,y + 2);

      g.setColor(Color.black);
      g.drawString(fMessages.lookup("textRailroadEnvironment"),cx1 + 200,y + 5);
    }
  }
}
