// ----------------------------------
// Filename      : SetScaleBox.java
// Created       : 14/12/1999
// Last modified : 09/02/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------

package mitrasim.modeler;

import java.awt.*;
import java.awt.event.*;
import mitrasim.resources.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class SetScaleBox extends Dialog implements WindowListener, ActionListener
{
  // internal datastructures
  private Resources   fResources;
  private Messages    fMessages;
  private int         fPixelDistance;
  private float       fRealDistance;
  private float       fScaleFactor;
  private final int   kPixelDistance = 1;
  private final int   kRealDistance  = 2;
  private CfgInputBox fCfgPixelDistance;
  private CfgInputBox fCfgRealDistance;
  private Label       fLabelScaleFactor;
  private boolean     fCanceled;

  public SetScaleBox(Frame parent, int PixelDistance)
  {
    super(parent);

    fResources = new Resources();
    fMessages = fResources.fMessages;
    fPixelDistance = PixelDistance;
    fRealDistance = 100.0f;
    fScaleFactor = fRealDistance / fPixelDistance;
    fCanceled = true;

    // adjust the windowlayout
    setFont(new Font(fMessages.lookup("textFontName"),Font.PLAIN,14));
    setResizable(false);
    Point Org = parent.getLocation();
    setLocation(Org.x + 20,Org.y + 60);
    setTitle(fMessages.lookup("textSetScaleDialogTitle"));
    setModal(true);
    setBackground(Color.lightGray);
    setForeground(Color.black);

    // define the controlpanels
    Panel SettingsPanel = new Panel();
    Panel ControlPanel = new Panel();
    SettingsPanel.setLayout(new GridLayout(4,3));
    ControlPanel.setLayout(new GridLayout(1,2));

    SettingsPanel.add(new Label(fMessages.lookup("textScalePixelDistance") + " = ",Label.RIGHT));
    fCfgPixelDistance = new CfgInputBox(kPixelDistance);
    SettingsPanel.add(fCfgPixelDistance);
    SettingsPanel.add(new Label(" " + fMessages.lookup("textPixelUnit"),Label.LEFT));

    SettingsPanel.add(new Label(fMessages.lookup("textScaleRealDistance") + " = ",Label.RIGHT));
    fCfgRealDistance = new CfgInputBox(kRealDistance);
    SettingsPanel.add(fCfgRealDistance);
    SettingsPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    SettingsPanel.add(new Label(fMessages.lookup("textScaleFactor") + " = ",Label.RIGHT));
    fLabelScaleFactor = new Label(String.valueOf(fScaleFactor));
    SettingsPanel.add(fLabelScaleFactor);
    SettingsPanel.add(new Label(" " + fMessages.lookup("textScaleFactorUnit"),Label.LEFT));

    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());
    SettingsPanel.add(new Label());

    Button button;
    button = new Button(fMessages.lookup("buttonCancel"));
    button.addActionListener(this);
    ControlPanel.add(button);

    button = new Button(fMessages.lookup("buttonOk"));
    button.addActionListener(this);
    ControlPanel.add(button);

    // add the controlpanel
    setLayout(new BorderLayout());
    add(SettingsPanel,"North");
    add(ControlPanel,"South");

    // add the windowlistener
    addWindowListener(this);
        
    pack();
    show();
  }

  public boolean canceled()
  {
    return fCanceled;
  }

  public float getScaleFactor()
  {
    return fScaleFactor;
  }

  private void updateAllFields()
  {
    fCfgPixelDistance.update();
    fCfgRealDistance.update();
    fLabelScaleFactor.setText(String.valueOf(fScaleFactor));
  }

  // methods from the WindowListener-interface
  public void windowOpened(WindowEvent e) { }
  public void windowClosing(WindowEvent e)
  {
    dispose();
  }
  public void windowClosed(WindowEvent e) { }
  public void windowIconified(WindowEvent e) { }
  public void windowDeiconified(WindowEvent e) { }
  public void windowActivated(WindowEvent e) { }
  public void windowDeactivated(WindowEvent e) { }

  // method from the ActionListener-interface
  public void actionPerformed(ActionEvent e)
  {
    String Command = e.getActionCommand();

    if (Command.equals(fMessages.lookup("buttonCancel"))) {
      windowClosing(null);
    }

    if (Command.equals(fMessages.lookup("buttonOk"))) {
      fCanceled = false;
      windowClosing(null);
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgInputBox extends TextField implements ActionListener
  {
    private int fInputBox;

    public CfgInputBox(int inputBox)
    {
      super();
      fInputBox = inputBox;
      update();
      addActionListener(this);
    }

    synchronized public void update()
    {
      if (fInputBox == kPixelDistance) {
        setText(String.valueOf(fPixelDistance));
      }
      else if (fInputBox == kRealDistance) {
        setText(String.valueOf(fRealDistance));
      }
    }

    // method from the ActionListener-interface
    public void actionPerformed(ActionEvent e)
    {
      int IntInput = 0;
      float FloatInput = 0.0f;

      if (fInputBox == kPixelDistance) {
        try {
          IntInput = (new Integer(e.getActionCommand())).intValue();
        }
        catch (NumberFormatException exc) {
          if (fInputBox == kPixelDistance) {
            IntInput = fPixelDistance;
          }
        }
      }
      else if (fInputBox == kRealDistance) {
        try {
          FloatInput = (new Float(e.getActionCommand())).floatValue();
        }
        catch (NumberFormatException exc) {
          if (fInputBox == kRealDistance) {
            FloatInput = fRealDistance;
          }
        }
      }

      if (fInputBox == kPixelDistance) {
        // don't allow negative and zero values
        if (IntInput == 0) {
          IntInput = 1;
        }
        else if (IntInput < 0) {
          IntInput = Math.abs(IntInput);
        }
        fPixelDistance = IntInput;
      }
      else if (fInputBox == kRealDistance) {
        // don't allow negative and zero values
        if (FloatInput == 0.0f) {
          FloatInput = 1.0f;
        }
        else if (FloatInput < 0.0f) {
          FloatInput = Math.abs(FloatInput);
        }
        fRealDistance = FloatInput;
      }

      fScaleFactor = fRealDistance / fPixelDistance;

      updateAllFields();
    }
  }
}
