// ----------------------------------
// Filename      : BoundingBox.java
// Created       : 07/11/1999
// Last modified : 24/04/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------

package mitrasim.network;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class BoundingBox
{
  public int     fMinX;
  public int     fMinY;
  public int     fMinZ;
  public int     fMaxX;
  public int     fMaxY;
  public int     fMaxZ;
  public boolean fUndefined;

  public BoundingBox()
  {
    fMinX = 0;
    fMinY = 0;
    fMinZ = 0;
    fMaxX = 0;
    fMaxY = 0;
    fMaxZ = 0;
    fUndefined = true;
  }

  public void calcUnion(BoundingBox boundingBox)
  {
    if (boundingBox.fUndefined) {
      // keep this bounding-box
      return;
    }

    if (fUndefined) {
      // the other bounding-box is not undefined so we use it
      fMinX = boundingBox.fMinX;
      fMinY = boundingBox.fMinY;
      fMinZ = boundingBox.fMinZ;
      fMaxX = boundingBox.fMaxX;
      fMaxY = boundingBox.fMaxY;
      fMaxZ = boundingBox.fMaxZ;
      fUndefined = false;
      return;
    }

    // calculate the union of the two bounding-boxes
    if (boundingBox.fMinX < fMinX) {
      fMinX = boundingBox.fMinX;
    }
    if (boundingBox.fMinY < fMinY) {
      fMinY = boundingBox.fMinY;
    }
    if (boundingBox.fMinZ < fMinZ) {
      fMinZ = boundingBox.fMinZ;
    }
    if (boundingBox.fMaxX > fMaxX) {
      fMaxX = boundingBox.fMaxX;
    }
    if (boundingBox.fMaxY > fMaxY) {
      fMaxY = boundingBox.fMaxY;
    }
    if (boundingBox.fMaxZ > fMaxZ) {
      fMaxZ = boundingBox.fMaxZ;
    }
    fUndefined = false;
  }

  public boolean contains2D(int x, int y)
  {
    return ((x >= fMinX) &&
            (y >= fMinY) &&
            (x <= fMaxX) &&
            (y <= fMaxY));
  }

  public boolean contains3D(int x, int y, int z)
  {
    return (contains2D(x,y) &&
            (z >= fMinZ) &&
            (z <= fMaxZ));
  }

  public void show()
  {
    System.out.println("(" +
                       fMinX + "," +
                       fMinY + "," +
                       fMinZ + "," +
                       fMaxX + "," +
                       fMaxY + "," +
                       fMaxZ + "," + fUndefined + ")");
  }

  public void cloneTo(BoundingBox duplicate)
  {
    duplicate.fMinX = fMinX;
    duplicate.fMinY = fMinY;
    duplicate.fMinZ = fMinZ;
    duplicate.fMaxX = fMaxX;
    duplicate.fMaxY = fMaxY;
    duplicate.fMaxZ = fMaxZ;
    duplicate.fUndefined = fUndefined;
  }
}
