// ----------------------------------
// Filename      : Protocol.java
// Created       : 07/03/2000
// Last modified : 02/05/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------

package mitrasim.server;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;
import mitrasim.resources.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class Protocol
{
  // the different protocols
  public static final int kBaseProtocol          = 0;
  public static final int kRemoteControlProtocol = 1;
  public static final int kVisualizerProtocol    = 2;

  // the different protocol-failures
  public static final int kProtocolOk               = 0;
  public static final int kInvalidProtocolRequested = 1;
  public static final int kProtocolAlreadyInUse     = 2;

  // internal datastructures
  protected Resources        fResources;
  protected Messages         fMessages;
  protected InetAddress      fIP;
  protected DataInputStream  fIn;
  protected DataOutputStream fOut;
  protected boolean          fIsOperational;

  public Protocol(InetAddress IP, DataInputStream in, DataOutputStream out)
  {
    fResources = new Resources();
    fMessages = fResources.fMessages;
    fIP = IP;
    fIn = in;
    fOut = out;
    fIsOperational = true;
  }

  public boolean isOperational()
  {
    return fIsOperational;
  }

  synchronized public void communicateWithClient()
  {
    try {
      // implement a simple echo-protocol
      byte data = fIn.readByte();
      showMessage(fMessages.lookup("messageEchoing"));
      fOut.writeByte(data);
      fOut.flush();
    }
    catch (IOException exc) {
      showMessage(fMessages.lookup("errorCommunicationFailure"));

      // get the date and time of disconnect
      Date CurrentDate = new Date();
      String Timestamp = DateFormat.getDateTimeInstance(DateFormat.SHORT,DateFormat.MEDIUM).format(CurrentDate);
      String ReportString = fMessages.lookup("messageLogDisconnect",
                                             Timestamp,
                                             String.valueOf(fIP),
                                             getProtocolAbbreviation());

      // log the disconnect
      fResources.fServerLog.addLine(ReportString);

      fIsOperational = false;
    }
  }

  public int getProtocol()
  {
    return kBaseProtocol;
  }

  public String getProtocolAbbreviation()
  {
    return fMessages.lookup("textBaseProtocolAbbreviation");;
  }

  public String translate(Messages messages)
  {
    return messages.lookup("textBaseProtocol");
  }

  protected void showMessage(String message)
  {
    System.out.println("[" + String.valueOf(fIP) + "] " + message);
  }

  protected void showMessageNoNewLine(String message)
  {
    System.out.print("[" + String.valueOf(fIP) + "] " + message);
  }
}
