// ----------------------------------
// Filename      : LoopDetector.java
// Created       : 21/05/2000
// Last modified : 23/06/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------

package mitrasim.simulator;

import java.util.*;
import mitrasim.network.*;
import mitrasim.simulator.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class LoopDetector
{
  public String   fName;
  public Node     fLocation;
  public int      fTimeForMeasurement;
  public int      fVehiclesPassed;
  public int      fCarsPassed;
  public int      fTrucksPassed;
  public float    fAverageSpeed;
  public float    fAverageFlowRate;
  public float    fAverageDensity;

  // internal datastructures
  private int    fTimeForMeasurementLeft;
  private int    fNrOfVehiclesMeasured;
  private float  fSumOfSpeedsMeasured;

  public LoopDetector(String Name, Node Location, int TimeForMeasurement)
  {
    fName = Name;
    fLocation = Location;
    fTimeForMeasurement = TimeForMeasurement;
    reset();
  }

  public void reset()
  {
    fVehiclesPassed = 0;
    fCarsPassed = 0;
    fTrucksPassed = 0;
    fAverageSpeed = 0.0f;
    fAverageFlowRate = 0.0f;
    fAverageDensity = 0.0f;

    // initiate a new measurement
    fTimeForMeasurementLeft = fTimeForMeasurement;
    fNrOfVehiclesMeasured = 0;
    fSumOfSpeedsMeasured = 0.0f;
  }

  // this method should be called every clocktick
  public void update()
  {
    // has the time for a measurement elapsed ?
    if (fTimeForMeasurementLeft <= 0) {

      // calculate the average speed
      if (fNrOfVehiclesMeasured == 0) {
        fAverageSpeed = 0.0f;
      }
      else {
        fAverageSpeed = fSumOfSpeedsMeasured / fNrOfVehiclesMeasured;
      }

      // calculate the average vehicles that have passed per hour (= 36000 clockticks)
      fAverageFlowRate = ((float) fNrOfVehiclesMeasured / fTimeForMeasurement) * 36000.0f;

      // calculate the average density from the fundamental
      // equation "density = flowrate / speed"
      if (fAverageSpeed != 0.0f) {
        fAverageDensity = fAverageFlowRate / fAverageSpeed;
      }
      else {
        fAverageDensity = 0.0f;
      }

      // initiate a new measurement
      fNrOfVehiclesMeasured = 0;
      fSumOfSpeedsMeasured = 0.0f;

      fTimeForMeasurementLeft = fTimeForMeasurement;
    }
    else {
      --fTimeForMeasurementLeft;
    }
  }

  public void countVehicle(Vehicle vehicle)
  {
    // increment the number of vehicles that has passed this loopdetector
    ++fVehiclesPassed;
    if (vehicle.fType == Vehicle.kCar) {
      ++fCarsPassed;
    }
    else {
      ++fTrucksPassed;
    }

    // record the speed of this vehicle
    fSumOfSpeedsMeasured += vehicle.fSpeed;
    ++fNrOfVehiclesMeasured;
  }
}
