// ----------------------------------
// Filename      : Messages.java
// Created       : 15/10/1999
// Last modified : 17/01/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------

package mitrasim.util;

import java.io.*;
import java.util.*;
import mitrasim.exceptions.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class Messages
{
  private FileInputStream fMessageFile;
  private Properties fMessageTable;

  public Messages(String messageFile) throws GeneralError
  {
    try {
      fMessageFile = new FileInputStream(messageFile);
      fMessageTable = new Properties();
      fMessageTable.load(fMessageFile);
    }
    catch (IOException exc) {
      throw (new GeneralError("Error, language file [" + messageFile + "] not found.",
                              GeneralError.kFileError));
    }
  }

  public String lookup(String messageKey)
  {
    return parseMessage(fMessageTable.getProperty(messageKey),"","","");
  }

  public String lookup(String messageKey, String param1)
  {
    return parseMessage(fMessageTable.getProperty(messageKey),param1,"","");
  }

  public String lookup(String messageKey, String param1, String param2)
  {
    return parseMessage(fMessageTable.getProperty(messageKey),param1,param2,"");
  }

  public String lookup(String messageKey, String param1, String param2, String param3)
  {
    return parseMessage(fMessageTable.getProperty(messageKey),param1,param2,param3);
  }

  private String parseMessage(String message, String param1, String param2, String param3)
  {
    if (message == null) {
      return "";
    }
    String fParsed = "";

    // parse the message
    for (int i = 0; i < message.length(); ++i) {
      String fToAdd = "";

      // was a parameter specified ?
      if (message.charAt(i) == '^') {
        ++i;
        // have we reached the end of the string ?
        if (i < message.length()) {
          // what parameter was specified ?
          if (message.charAt(i) == '1') {
            fToAdd = param1;
          }
          else if (message.charAt(i) == '2') {
            fToAdd = param2;
          }
          else if (message.charAt(i) == '3') {
            fToAdd = param3;
          }
          else if (message.charAt(i) == '^') {
            fToAdd = "^";
          }	    
        }
      }
      else {
        // no parameter was specified
        fToAdd = String.valueOf(message.charAt(i));
      }

      fParsed = fParsed.concat(fToAdd);
    }

    return fParsed;
  }
}
