// -----------------------------------
// Filename      : ViewOptionsBox.java
// Created       : 03/11/1999
// Last modified : 19/01/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// -----------------------------------

package mitrasim.viewer;

import java.awt.*;
import java.awt.event.*;
import mitrasim.engine3d.*;
import mitrasim.resources.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class ViewOptionsBox extends Dialog implements WindowListener, ActionListener
{
  // internal datastructures
  private Resources   fResources;
  private Messages    fMessages;

  private final int   kProjectionMode      = 1;
  private final int   kShowIDs             = 2;
  private final int   kShowNodes           = 3;
  private final int   kShowLinks           = 4;
  private final int   kRenderSplines       = 5;
  private final int   kSplineInterpolation = 6;
  private final int   kShowEnvironment     = 7;
  private final int   kOvalInterpolation   = 8;
  private final int   kShowBoundingBox     = 9;
  private final int   kShowAxes            = 10;
  private final int   kShowGrid            = 11;
  private final int   kGridType            = 12;
  private final int   kGridFrequency       = 13;
  private final int   kGridHeight          = 14;
  private final int   kDistanceStepSize    = 15;
  private final int   kRotateStepSize      = 16;

  private CfgItemList fCfgProjectionMode;
  private CfgCheckBox fCfgShowIDs;
  private CfgCheckBox fCfgShowNodes;
  private CfgCheckBox fCfgShowLinks;
  private CfgCheckBox fCfgRenderSplines;
  private CfgInputBox fCfgSplineInterpolation;
  private CfgCheckBox fCfgShowEnvironment;
  private CfgInputBox fCfgOvalInterpolation;
  private CfgCheckBox fCfgShowBoundingBox;
  private CfgCheckBox fCfgShowAxes;
  private CfgCheckBox fCfgShowGrid;
  private CfgItemList fCfgGridType;
  private CfgInputBox fCfgGridFrequency;
  private CfgInputBox fCfgGridHeight;
  private CfgInputBox fCfgDistanceStepSize;
  private CfgInputBox fCfgRotateStepSize;

  private boolean     fOldPerspectiveMode;
  private boolean     fOldShowIDs;
  private boolean     fOldShowNodes;
  private boolean     fOldShowLinks;
  private boolean     fOldRenderSplines;
  private int         fOldSplineInterpolation;
  private boolean     fOldShowEnvironment;
  private int         fOldOvalInterpolation;
  private boolean     fOldShowBoundingBox;
  private boolean     fOldShowAxes;
  private boolean     fOldShowGrid;
  private boolean     fOldPointGridType;
  private int         fOldGridFrequency;
  private int         fOldGridHeight;
  private int         fOldDistanceStepSize;
  private int         fOldRotateStepSize;

  private boolean     fCanceled;

  public ViewOptionsBox(Frame parent)
  {
    super(parent);

    fResources = new Resources();
    fMessages = fResources.fMessages;
    fOldPerspectiveMode     = fResources.fnvPerspectiveMode;
    fOldShowIDs             = fResources.fnvShowIDs;
    fOldShowNodes           = fResources.fnvShowNodes;
    fOldShowLinks           = fResources.fnvShowLinks;
    fOldRenderSplines       = fResources.fnvRenderSplines;
    fOldSplineInterpolation = fResources.fnvSplineInterpolation;
    fOldShowEnvironment     = fResources.fnvShowEnvironment;
    fOldOvalInterpolation   = fResources.fnvOvalInterpolation;
    fOldShowBoundingBox     = fResources.fnvShowBoundingBox;
    fOldShowAxes            = fResources.fnvShowAxes;
    fOldShowGrid            = fResources.fnvShowGrid;
    fOldPointGridType       = fResources.fnvPointGridType;
    fOldGridFrequency       = fResources.fnvGridFrequency;
    fOldGridHeight          = fResources.fnvGridHeight;
    fOldDistanceStepSize    = fResources.fnvDistanceStepSize;
    fOldRotateStepSize      = fResources.fnvRotateStepSize;
    fCanceled = true;

    // adjust the windowlayout
    setFont(new Font(fMessages.lookup("textFontName"),Font.PLAIN,14));
    setResizable(false);
    Point Org = parent.getLocation();
    setLocation(Org.x + 20,Org.y + 60);
    setTitle(fMessages.lookup("textViewOptionsBoxTitle"));
    setModal(true);
    setBackground(Color.lightGray);
    setForeground(Color.black);

    // create the configuration-panel
    Panel CfgPanel = new Panel();
    CfgPanel.setLayout(new GridLayout(19,3));

    CfgPanel.add(new Label(fMessages.lookup("textProjectionMode") + " : ",Label.RIGHT));
    fCfgProjectionMode = new CfgItemList(kProjectionMode);
    CfgPanel.add(fCfgProjectionMode);
    CfgPanel.add(new Label());

    CfgPanel.add(new Label(fMessages.lookup("textShowIDs") + " : ",Label.RIGHT));
    fCfgShowIDs = new CfgCheckBox(kShowIDs);
    CfgPanel.add(fCfgShowIDs);
    CfgPanel.add(new Label());

    CfgPanel.add(new Label(fMessages.lookup("textShowNodes") + " : ",Label.RIGHT));
    fCfgShowNodes = new CfgCheckBox(kShowNodes);
    CfgPanel.add(fCfgShowNodes);
    CfgPanel.add(new Label());

    CfgPanel.add(new Label(fMessages.lookup("textShowLinks") + " : ",Label.RIGHT));
    fCfgShowLinks = new CfgCheckBox(kShowLinks);
    CfgPanel.add(fCfgShowLinks);
    CfgPanel.add(new Label());

    CfgPanel.add(new Label(fMessages.lookup("textRenderSplines") + " : ",Label.RIGHT));
    fCfgRenderSplines = new CfgCheckBox(kRenderSplines);
    CfgPanel.add(fCfgRenderSplines);
    CfgPanel.add(new Label());

    CfgPanel.add(new Label(fMessages.lookup("textSplineInterpolation") + " : ",Label.RIGHT));
    fCfgSplineInterpolation = new CfgInputBox(kSplineInterpolation);
    CfgPanel.add(fCfgSplineInterpolation);
    CfgPanel.add(new Label(" " + fMessages.lookup("textPoints"),Label.LEFT));

    CfgPanel.add(new Label());
    CfgPanel.add(new Label());
    CfgPanel.add(new Label());

    CfgPanel.add(new Label(fMessages.lookup("textShowEnvironment") + " : ",Label.RIGHT));
    fCfgShowEnvironment = new CfgCheckBox(kShowEnvironment);
    CfgPanel.add(fCfgShowEnvironment);
    CfgPanel.add(new Label());

    CfgPanel.add(new Label(fMessages.lookup("textOvalInterpolation") + " : ",Label.RIGHT));
    fCfgOvalInterpolation = new CfgInputBox(kOvalInterpolation);
    CfgPanel.add(fCfgOvalInterpolation);
    CfgPanel.add(new Label(" " + fMessages.lookup("textPoints"),Label.LEFT));

    CfgPanel.add(new Label(fMessages.lookup("textShowBoundingBox") + " : ",Label.RIGHT));
    fCfgShowBoundingBox = new CfgCheckBox(kShowBoundingBox);
    CfgPanel.add(fCfgShowBoundingBox);
    CfgPanel.add(new Label());

    CfgPanel.add(new Label(fMessages.lookup("textShowAxes") + " : ",Label.RIGHT));
    fCfgShowAxes = new CfgCheckBox(kShowAxes);
    CfgPanel.add(fCfgShowAxes);
    CfgPanel.add(new Label());

    CfgPanel.add(new Label(fMessages.lookup("textShowGrid") + " : ",Label.RIGHT));
    fCfgShowGrid = new CfgCheckBox(kShowGrid);
    CfgPanel.add(fCfgShowGrid);
    CfgPanel.add(new Label());

    CfgPanel.add(new Label(fMessages.lookup("textGridType") + " : ",Label.RIGHT));
    fCfgGridType = new CfgItemList(kGridType);
    CfgPanel.add(fCfgGridType);
    CfgPanel.add(new Label());

    CfgPanel.add(new Label(fMessages.lookup("textGridFrequency") + " : ",Label.RIGHT));
    fCfgGridFrequency = new CfgInputBox(kGridFrequency);
    CfgPanel.add(fCfgGridFrequency);
    CfgPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    CfgPanel.add(new Label(fMessages.lookup("textGridHeight") + " : ",Label.RIGHT));
    fCfgGridHeight = new CfgInputBox(kGridHeight);
    CfgPanel.add(fCfgGridHeight);
    CfgPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    CfgPanel.add(new Label());
    CfgPanel.add(new Label());
    CfgPanel.add(new Label());

    CfgPanel.add(new Label(fMessages.lookup("textDistanceStepSize") + " : ",Label.RIGHT));
    fCfgDistanceStepSize = new CfgInputBox(kDistanceStepSize);
    CfgPanel.add(fCfgDistanceStepSize);
    CfgPanel.add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    CfgPanel.add(new Label(fMessages.lookup("textRotateStepSize") + " : ",Label.RIGHT));
    fCfgRotateStepSize = new CfgInputBox(kRotateStepSize);
    CfgPanel.add(fCfgRotateStepSize);
    CfgPanel.add(new Label(" " + fMessages.lookup("textRotateUnit"),Label.LEFT));

    // create the control-panel
    Panel ControlPanel = new Panel();
    ControlPanel.setLayout(new GridLayout(1,2));

    Button button;
    button = new Button(fMessages.lookup("buttonCancel"));
    button.addActionListener(this);
    ControlPanel.add(button);

    button = new Button(fMessages.lookup("buttonOk"));
    button.addActionListener(this);
    ControlPanel.add(button);

    setLayout(new BorderLayout());
    add("North",CfgPanel);
    add("South",ControlPanel);

    // add the windowlistener
    addWindowListener(this);
        
    pack();
    show();
  }

  private void update()
  {
    fCfgProjectionMode.update();
    fCfgShowIDs.update();
    fCfgShowNodes.update();
    fCfgShowLinks.update();
    fCfgRenderSplines.update();
    fCfgSplineInterpolation.update();
    fCfgShowEnvironment.update();
    fCfgOvalInterpolation.update();
    fCfgShowBoundingBox.update();
    fCfgShowAxes.update();
    fCfgShowGrid.update();
    fCfgGridType.update();
    fCfgGridFrequency.update();
    fCfgGridHeight.update();
    fCfgDistanceStepSize.update();
    fCfgRotateStepSize.update();
  }

  // methods from the WindowListener-interface
  public void windowOpened(WindowEvent e) { }
  public void windowClosing(WindowEvent e)
  {
    if (fCanceled) {
      // restore all the old values
      fResources.fnvPerspectiveMode     = fOldPerspectiveMode;
      fResources.fnvShowIDs             = fOldShowIDs;
      fResources.fnvShowNodes           = fOldShowNodes;
      fResources.fnvShowLinks           = fOldShowLinks;
      fResources.fnvRenderSplines       = fOldRenderSplines;
      fResources.fnvSplineInterpolation = fOldSplineInterpolation;
      fResources.fnvShowEnvironment     = fOldShowEnvironment;
      fResources.fnvOvalInterpolation   = fOldOvalInterpolation;
      fResources.fnvShowBoundingBox     = fOldShowBoundingBox;
      fResources.fnvShowAxes            = fOldShowAxes;
      fResources.fnvShowGrid            = fOldShowGrid;
      fResources.fnvPointGridType       = fOldPointGridType;
      fResources.fnvGridFrequency       = fOldGridFrequency;
      fResources.fnvGridHeight          = fOldGridHeight;
      fResources.fnvDistanceStepSize    = fOldDistanceStepSize;
      fResources.fnvRotateStepSize      = fOldRotateStepSize;
    }
    dispose();
  }
  public void windowClosed(WindowEvent e) { }
  public void windowIconified(WindowEvent e) { }
  public void windowDeiconified(WindowEvent e) { }
  public void windowActivated(WindowEvent e) { }
  public void windowDeactivated(WindowEvent e) { }

  // method from the ActionListener-interface
  public void actionPerformed(ActionEvent e)
  {
    String Command = e.getActionCommand();

    if (Command.equals(fMessages.lookup("buttonCancel"))) {
      windowClosing(null);
    }

    if (Command.equals(fMessages.lookup("buttonOk"))) {
      fCanceled = false;
      windowClosing(null);
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgCheckBox extends Checkbox implements ItemListener
  {
    private int fCheckBox;

    public CfgCheckBox(int checkBox)
    {
      super();
      fCheckBox = checkBox;
      addItemListener(this);
      update();
    }

    synchronized public void update()
    {
      if (fCheckBox == kShowIDs) {
        setState(fResources.fnvShowIDs);
        if (fResources.fnvShowIDs) {
          setLabel("(" + fMessages.lookup("textYes") + ")");
        }
        else {
          setLabel("(" + fMessages.lookup("textNo") + ")");
        }
      }
      else if (fCheckBox == kShowNodes) {
        setState(fResources.fnvShowNodes);
        if (fResources.fnvShowNodes) {
          setLabel("(" + fMessages.lookup("textYes") + ")");
        }
        else {
          setLabel("(" + fMessages.lookup("textNo") + ")");
        }
      }
      else if (fCheckBox == kShowLinks) {
        setState(fResources.fnvShowLinks);
        if (fResources.fnvShowLinks) {
          setLabel("(" + fMessages.lookup("textYes") + ")");
        }
        else {
          setLabel("(" + fMessages.lookup("textNo") + ")");
        }
      }
      else if (fCheckBox == kRenderSplines) {
        setState(fResources.fnvRenderSplines);
        if (fResources.fnvRenderSplines) {
          setLabel("(" + fMessages.lookup("textYes") + ")");
        }
        else {
          setLabel("(" + fMessages.lookup("textUseStraightLines") + ")");
        }
      }
      else if (fCheckBox == kShowEnvironment) {
        setState(fResources.fnvShowEnvironment);
        if (fResources.fnvShowEnvironment) {
          setLabel("(" + fMessages.lookup("textYes") + ")");
        }
        else {
          setLabel("(" + fMessages.lookup("textNo") + ")");
        }
      }
      else if (fCheckBox == kShowBoundingBox) {
        setState(fResources.fnvShowBoundingBox);
        if (fResources.fnvShowBoundingBox) {
          setLabel("(" + fMessages.lookup("textYes") + ")");
        }
        else {
          setLabel("(" + fMessages.lookup("textNo") + ")");
        }
      }
      else if (fCheckBox == kShowAxes) {
        setState(fResources.fnvShowAxes);
        if (fResources.fnvShowAxes) {
          setLabel("(" + fMessages.lookup("textYes") + ")");
        }
        else {
          setLabel("(" + fMessages.lookup("textNo") + ")");
        }
      }
      else if (fCheckBox == kShowGrid) {
        setState(fResources.fnvShowGrid);
        if (fResources.fnvShowGrid) {
          setLabel("(" + fMessages.lookup("textYes") + ")");
        }
        else {
          setLabel("(" + fMessages.lookup("textNo") + ")");
        }
      }
    }

    // method from the ItemListener-interface
    public void itemStateChanged(ItemEvent e)
    {
      if (fCheckBox == kShowIDs) {
        fResources.fnvShowIDs = !fResources.fnvShowIDs;
      }
      else if (fCheckBox == kShowNodes) {
        fResources.fnvShowNodes = !fResources.fnvShowNodes;
      }
      else if (fCheckBox == kShowLinks) {
        fResources.fnvShowLinks = !fResources.fnvShowLinks;
      }
      else if (fCheckBox == kRenderSplines) {
        fResources.fnvRenderSplines = !fResources.fnvRenderSplines;
      }
      else if (fCheckBox == kShowEnvironment) {
        fResources.fnvShowEnvironment = !fResources.fnvShowEnvironment;
      }
      else if (fCheckBox == kShowBoundingBox) {
        fResources.fnvShowBoundingBox = !fResources.fnvShowBoundingBox;
      }
      else if (fCheckBox == kShowAxes) {
        fResources.fnvShowAxes = !fResources.fnvShowAxes;
      }
      else if (fCheckBox == kShowGrid) {
        fResources.fnvShowGrid = !fResources.fnvShowGrid;
      }

      update();
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgInputBox extends TextField implements ActionListener
  {
    private int fInputBox;

    public CfgInputBox(int inputBox)
    {
      super();
      fInputBox = inputBox;
      update();
      addActionListener(this);
    }

    synchronized public void update()
    {
      if (fInputBox == kSplineInterpolation) {
        setText(String.valueOf(fResources.fnvSplineInterpolation));
      }
      else if (fInputBox == kOvalInterpolation) {
        setText(String.valueOf(fResources.fnvOvalInterpolation));
      }
      else if (fInputBox == kGridFrequency) {
        setText(String.valueOf(fResources.fnvGridFrequency));
      }
      else if (fInputBox == kGridHeight) {
        setText(String.valueOf(fResources.fnvGridHeight));
      }
      else if (fInputBox == kDistanceStepSize) {
        setText(String.valueOf(fResources.fnvDistanceStepSize));
      }
      else if (fInputBox == kRotateStepSize) {
        setText(String.valueOf(fResources.fnvRotateStepSize));
      }
    }

    // method from the ActionListener-interface
    public void actionPerformed(ActionEvent e)
    {
      int NewInput = 0;
      try {
        NewInput = (new Integer(e.getActionCommand())).intValue();
      }
      catch (NumberFormatException exc) {
        if (fInputBox == kSplineInterpolation) {
          NewInput = fResources.fnvSplineInterpolation;
        }
        else if (fInputBox == kOvalInterpolation) {
          NewInput = fResources.fnvOvalInterpolation;
        }
        else if (fInputBox == kGridFrequency) {
          NewInput = fResources.fnvGridFrequency;
        }
        else if (fInputBox == kGridHeight) {
          NewInput = fResources.fnvGridHeight;
        }
        else if (fInputBox == kDistanceStepSize) {
          NewInput = fResources.fnvDistanceStepSize;
        }
        else if (fInputBox == kRotateStepSize) {
          NewInput = fResources.fnvRotateStepSize;
        }
      }

      if (fInputBox == kSplineInterpolation) {
        // don't allow too small values
        if (NewInput < Resources.knvMinSplineInterpolation) {
          NewInput = Resources.knvMinSplineInterpolation;
        }
        fResources.fnvSplineInterpolation = NewInput;
      }
      else if (fInputBox == kOvalInterpolation) {
        // don't allow too small values
        if (NewInput < Resources.knvMinOvalInterpolation) {
          NewInput = Resources.knvMinOvalInterpolation;
        }
        fResources.fnvOvalInterpolation = NewInput;
      }
      else if (fInputBox == kGridFrequency) {
        // don't allow too small values
        if (NewInput < Resources.knvMinGridFrequency) {
          NewInput = Resources.knvMinGridFrequency;
        }
        fResources.fnvGridFrequency = NewInput;
      }
      else if (fInputBox == kGridHeight) {
        fResources.fnvGridHeight = NewInput;
      }
      if (fInputBox == kDistanceStepSize) {
        // don't allow negative values
        if (NewInput < 0) {
          NewInput = Math.abs(NewInput);
        }
        // don't allow a zero-stepsize
        if (NewInput == 0) {
          NewInput = 1;
        }
        fResources.fnvDistanceStepSize = NewInput;
      }
      else if (fInputBox == kRotateStepSize) {
        // don't allow negative values
        if (NewInput < 0) {
          NewInput = Math.abs(NewInput);
        }
        // don't allow a zero-stepsize
        if (NewInput == 0) {
          NewInput = 1;
        }
        if (NewInput > 180) {
          NewInput = 180;
        }
        fResources.fnvRotateStepSize = NewInput;
      }

      update();
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgItemList extends Choice implements ItemListener
  {
    private int fItemList;

    public CfgItemList(int itemList)
    {
      super();
      fItemList = itemList;
      if (fItemList == kProjectionMode) {
        add(fResources.fMessages.lookup("textPerspectiveProjection"));
        add(fResources.fMessages.lookup("textParallelProjection"));
      }
      else if (fItemList == kGridType) {
        add(fResources.fMessages.lookup("textPointGridType"));
        add(fResources.fMessages.lookup("textLineGridType"));
      }
      update();
      addItemListener(this);
    }

    synchronized public void update()
    {
      if (fItemList == kProjectionMode) {
        if (fResources.fnvPerspectiveMode) {
          select(fResources.fMessages.lookup("textPerspectiveProjection"));
        }
        else {
          select(fResources.fMessages.lookup("textParallelProjection"));
        }
      }
      else if (fItemList == kGridType) {
        if (fResources.fnvPointGridType) {
          select(fResources.fMessages.lookup("textPointGridType"));
        }
        else {
          select(fResources.fMessages.lookup("textLineGridType"));
        }
      }
    }

    // method from the ItemListener-interface
    public void itemStateChanged(ItemEvent e)
    {
      if (fItemList == kProjectionMode) {
        String ProjectionMode = (String) e.getItem();
        if (ProjectionMode.equalsIgnoreCase(fResources.fMessages.lookup("textPerspectiveProjection"))) {
          fResources.fnvPerspectiveMode = true;
        }
        else {
          fResources.fnvPerspectiveMode = false;
        }
      }
      else if (fItemList == kGridType) {
        String PointGrid = (String) e.getItem();
        if (PointGrid.equalsIgnoreCase(fResources.fMessages.lookup("textPointGridType"))) {
          fResources.fnvPointGridType = true;
        }
        else {
          fResources.fnvPointGridType = false;
        }
      }
    }
  }
}
