// ----------------------------------
// Filename      : ViewPointBox.java
// Created       : 03/11/1999
// Last modified : 05/03/2000
// Target        : Java VM
// Author        : Sven Maerivoet
//
// All rights reserved.
// Universitaire Instelling Antwerpen
// ----------------------------------

package mitrasim.viewer;

import java.awt.*;
import java.awt.event.*;
import mitrasim.engine3d.*;
import mitrasim.resources.*;
import mitrasim.util.*;

/** @author Sven Maerivoet<BR>
    All rights reserved.<BR>
    Universitaire Instelling Antwerpen
*/
public class ViewPointBox extends Dialog implements WindowListener, ActionListener
{
  // internal datastructures
  private Resources   fResources;
  private Messages    fMessages;
  private final int   kXPosition = 0;
  private final int   kYPosition = 1;
  private final int   kZPosition = 2;

  private CfgInputBox fCfgXPosition;
  private CfgInputBox fCfgYPosition;
  private CfgInputBox fCfgZPosition;

  private Point3D     fOldViewPoint;

  private boolean     fCanceled;

  public ViewPointBox(Frame parent)
  {
    super(parent);

    fResources = new Resources();
    fMessages = fResources.fMessages;
    fOldViewPoint = new Point3D();
    fOldViewPoint.fX = fResources.fnvViewPoint.fX;
    fOldViewPoint.fY = fResources.fnvViewPoint.fY;
    fOldViewPoint.fZ = fResources.fnvViewPoint.fZ;
    fCanceled = true;

    // adjust the windowlayout
    setFont(new Font(fMessages.lookup("textFontName"),Font.PLAIN,14));
    setResizable(false);
    Point Org = parent.getLocation();
    setLocation(Org.x + 20,Org.y + 60);
    setTitle(fMessages.lookup("textViewPointBoxTitle"));
    setModal(true);
    setBackground(Color.lightGray);
    setForeground(Color.black);

    setLayout(new GridLayout(5,3));

    add(new Label("X = ",Label.RIGHT));
    fCfgXPosition = new CfgInputBox(kXPosition);
    add(fCfgXPosition);
    add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    add(new Label("Y = ",Label.RIGHT));
    fCfgYPosition = new CfgInputBox(kYPosition);
    add(fCfgYPosition);
    add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    add(new Label("Z = ",Label.RIGHT));
    fCfgZPosition = new CfgInputBox(kZPosition);
    add(fCfgZPosition);
    add(new Label(" " + fMessages.lookup("textDistanceUnit"),Label.LEFT));

    add(new Label());
    add(new Label());
    add(new Label());

    Button ControlButton;
    ControlButton = new Button(fMessages.lookup("buttonOrigin"));
    ControlButton.addActionListener(this);
    add(ControlButton);

    ControlButton = new Button(fMessages.lookup("buttonCancel"));
    ControlButton.addActionListener(this);
    add(ControlButton);

    ControlButton = new Button(fMessages.lookup("buttonOk"));
    ControlButton.addActionListener(this);
    add(ControlButton);

    // add the windowlistener
    addWindowListener(this);
        
    pack();
    show();
  }

  private void update()
  {
    fCfgXPosition.update();
    fCfgYPosition.update();
    fCfgZPosition.update();
  }

  // methods from the WindowListener-interface
  public void windowOpened(WindowEvent e) { }
  public void windowClosing(WindowEvent e)
  {
    if (fCanceled) {
      // restore the old position of the viewpoint
      fResources.fnvViewPoint.fX = fOldViewPoint.fX;
      fResources.fnvViewPoint.fY = fOldViewPoint.fY;
      fResources.fnvViewPoint.fZ = fOldViewPoint.fZ;
    }
    dispose();
  }
  public void windowClosed(WindowEvent e) { }
  public void windowIconified(WindowEvent e) { }
  public void windowDeiconified(WindowEvent e) { }
  public void windowActivated(WindowEvent e) { }
  public void windowDeactivated(WindowEvent e) { }

  // method from the ActionListener-interface
  public void actionPerformed(ActionEvent e)
  {
    String Command = e.getActionCommand();

    if (Command.equals(fMessages.lookup("buttonOrigin"))) {
      fResources.fnvViewPoint.set(0.0f,0.0f,0.0f);
      update();
    }

    if (Command.equals(fMessages.lookup("buttonCancel"))) {
      windowClosing(null);
    }

    if (Command.equals(fMessages.lookup("buttonOk"))) {
      fCanceled = false;
      windowClosing(null);
    }
  }

  /** @author Sven Maerivoet<BR>
      All rights reserved.<BR>
      Universitaire Instelling Antwerpen
  */  
  private class CfgInputBox extends TextField implements ActionListener
  {
    private int fAxis;

    public CfgInputBox(int axis)
    {
      super();
      fAxis = axis;
      update();
      addActionListener(this);
    }

    synchronized public void update()
    {
      if (fAxis == kXPosition) {
        setText(String.valueOf(fResources.fnvViewPoint.fX));
      }
      else if (fAxis == kYPosition) {
        setText(String.valueOf(fResources.fnvViewPoint.fY));
      }
      else if (fAxis == kZPosition) {
        setText(String.valueOf(fResources.fnvViewPoint.fZ));
      }
    }

    // method from the ActionListener-interface
    public void actionPerformed(ActionEvent e)
    {
      float Position = 0.0f;
      try {
        Position = (new Float(e.getActionCommand())).floatValue();
      }
      catch (NumberFormatException exc) {
        if (fAxis == kXPosition) {
          Position = fResources.fnvViewPoint.fX;
        }
        else if (fAxis == kYPosition) {
          Position = fResources.fnvViewPoint.fY;
        }
        else if (fAxis == kZPosition) {
          Position = fResources.fnvViewPoint.fZ;
        }
      }

      if (fAxis == kXPosition) {
        fResources.fnvViewPoint.fX = Position;
      }
      else if (fAxis == kYPosition) {
        fResources.fnvViewPoint.fY = Position;
      }
      else if (fAxis == kZPosition) {
        fResources.fnvViewPoint.fZ = Position;
      }

      update();
    }
  }
}
